#!/usr/bin/perl

use strict;
use warnings;

use Chart::Clicker;
use Chart::Clicker::Data::Series;
use Chart::Clicker::Data::DataSet;
use Time::HiRes qw(time);

my @lengths = ( 0 .. 27 );
my @durations;

for my $length ( @lengths ) {
    my $string = "a" x $length;

    my $start    = time;
    $string      =~ /(.*){1,32000}[bc]/;
    my $duration = time - $start;

    push @durations, $duration;
}

my $series = Chart::Clicker::Data::Series->new(
    keys   => \@lengths,
    values => \@durations,
);

my $ds = Chart::Clicker::Data::DataSet->new( series => [$series] );

my $cc = Chart::Clicker->new(width => 1000, height => 500, format => 'svg');
$cc->title->text('Execution times \'("a" x $length) =~ /(.*){1,32000}[bc]/\'');
$cc->title->padding->bottom(5);
$cc->add_to_datasets( $ds );

my $defctx = $cc->get_context('default');

$defctx->range_axis->label('time');
$defctx->domain_axis->label('length');
$defctx->domain_axis->tick_values( [0,5,10,15,20,25] );
$defctx->domain_axis->tick_labels( [0,5,10,15,20,25] );
$defctx->renderer->brush->width(2);

$cc->write_output('redos-graph.svg');
