# Perl in 2022

---

# Perl in 2022

* overlaps with Paul's talk

<https://docs.google.com/presentation/d/1UGuXjf1fid8GSpdAsY4utzp-rVPL5wUAZmpUUsiLNaQ>

------

# About me

* Renée Bäcker
  * Owner of <a href="https://perl-services.de">Perl-Services.de</a> (<a href="https://metacpan.org/author/PERLSRVDE">PERLSRVDE @ cpan</a>)
  * <a href="https://perl-academy.de">Perl-Academy.de</a>
  * <a href="https://metacpan.org/author/RENEEB">RENEEB @ cpan</a>
  * Interests: Perl, IT-Security

------

# About this talk

- one stable Perl release each year
- new features in Perl 5.34
  - which was released in 2021
- new features in Perl 5.36
  - will be released in 2022
  - things might change...

------

# experimental try/catch

* who likes exception handling in Perl?
* use (block) `eval`?

```perl
eval {
    die "Help!";
};

if ( $@ ) {
    warn "An error occured: $@";
}
```

---

# experimental try/catch

Loads of try/catch modules on CPAN

- Try::Tiny
- Syntax::Keyword::Try
- Try::Catch
- Try, Nice::Try, Try::Lite, Try::Harder, Try::Tiny::Tiny, ...

<http://mvp.kablamo.org/cpan/exceptions/>

---

# experimental try/catch

Cons:
- some are "slow"
- some have several dependencies
- which one to use?
- actively maintained?

---

# experimental try/catch

```perl [1-2|4-6|7-9|]
use feature 'try';
no warnings 'experimental::try';

try {
    die 'This died';
}
catch ( $error ) {
    warn "The code inside 'try' died with error: '$error'";
}
```

```
$ perl try_catch.pl 
The code inside 'try' died with error: 'This died at try_catch.pl line 5.
' at try_catch.pl line 8.
```

---

# experimental try/catch

```perl [6-8|9-12|]
use feature 'try';
no warnings 'experimental::try';

package ErrorObject { use Moo; has message => ( is => 'ro' ) }

try {
    die ErrorObject->new( message => 'An error occured' );
}
catch ( $error ) {
    warn sprintf "The code inside 'try' died with error: '%s'",
        $error->message;
}
```

```
The code inside 'try' died with error: 'An error occured' at objects.pl line 10.
```

---

# experimental try/catch/finally (5.36)

```perl [9,12,13|14-16|]
use feature 'try';
no warnings 'experimental::try';

package ErrorObject { use Moo; has message => ( is => 'ro' ) }

try {
    die ErrorObject->new( message => 'An error occured' );
}
catch ( $error ) {
    warn sprintf "The code inside 'try' died with error: '%s'",
        $error->message;
    exit;
}
finally {
    print "clean up...";
}
```

```
$ perl finally.pl 
The code inside 'try' died with error: 'An error occured' at finally.pl line 10.
clean up...
```

------

# Regex: {,n} is now allowed

matches up to _n_ occurences of the term in front of _{...}_

- `{6}` &mdash; matches exactly 6 occurences
- `{1,6}` &mdash; matches 1 up to 6 occurences
- `{0,6}` &mdash; matches up to 6 occurences
- `{,6}` &mdash; matches up to 6 occurences

------

# New octal syntax

* you already know `0b....` for binary and `0x...` for hexadecimals
* now you can use `0o....` for octals

------

# `use v5.35` changes

* `strict` was enabled long time ago, it now enables `warnings`, too

```perl [|]
use v5.35;

my $s = 'test';
$s += 1;
```

```
$ perl use_v535.pl 
Argument "test" isn't numeric in addition (+) at use_v535.pl line 4.
```

---

# `use v5.35` changes

* bareword filehandles and
* indirect method invocations and
* multidimensional are fatal now

```perl [3-5|]
use v5.35;

open FH, '>', '/tmp/test' or die $!;
print FH 'test';
close FH;
```

```
$ perl filehandles.pl 
Bareword filehandle "FH" not allowed under 'no feature "bareword_filehandles"' at filehandles.pl line 3.
Bareword filehandle "FH" not allowed under 'no feature "bareword_filehandles"' at filehandles.pl line 4.
Bareword filehandle "FH" not allowed under 'no feature "bareword_filehandles"' at filehandles.pl line 5.
Execution of filehandles.pl aborted due to compilation errors.
```

---

# `use v5.35` changes

* bareword filehandles are fatal now
  * can be enabled with `use feature 'bareword_filehandles';`
* Old code still runs **without** `use v5.35`

---

# `use v5.35` changes

* bareword filehandles and
* indirect method invocations and
* multidimensional are fatal now

```perl [5|]
use v5.35;

{ package Class; sub new { return bless {}, shift } }

my $obj = new Class;
```

```
$ perl indirect.pl 
Bareword found where operator expected at indirect.pl line 5, near "new Class"
        (Do you need to predeclare new?)
syntax error at indirect.pl line 5, near "new Class"
```

---

# `use v5.35` changes

* indirect method invocations are fatal now
  * can be enabled with `use feature 'indirect';`
* Old code still runs **without** `use v5.35`

---

# `use v5.35` changes

* bareword filehandles and
* indirect method invocations and
* multidimensional are fatal now

```perl
my %x;

$x{1,'talk'} = 'Perl Features';
print $x{1,'talk'};
```

---

# `use v5.35` changes

* bareword filehandles and
* indirect method invocations and
* multidimensional are fatal now

```perl
use v5.35;

my %x;

$x{1,'talk'} = 'Perl Features';
print $x{1,'talk'};
```

```
$ perl v535/multidimensional.pl 
Multidimensional hash lookup is disabled at v535/multidimensional.pl line 5, near "'talk'}"
Multidimensional hash lookup is disabled at v535/multidimensional.pl line 6, near "'talk'}"
Execution of test.pl aborted due to compilation errors.
```

---

# `use v5.35` changes

* multidimensional is fatal now
  * can be enabled with `use feature 'multidimensional';`
* Old code still runs **without** `use v5.35`

---

# `use v5.35` changes

* _switch_ is gone from the feature bundle

------

# De-_experiment_-ed features:

* `isa` operator
* signatures
  * but using `@_` in signatured subroutines triggers a warning

---

# De-_experiment_-ed features:

* `isa` operator

```perl [1,2|4,6|8|9|11,12]
use v5.35;
# or use feature 'isa';

{ package T; sub new { return bless {}, shift } };

my $obj = T->new;

if ( $obj isa T )   { print "isa T (1) \n" }
if ( $obj isa "T" ) { print "isa T (2) \n" }

my $name = "T";
if ( $obj isa $name ) { print "isa T (3) \n" }
```

```
$ perl deexperimented/isa.pl 
isa T (1) 
isa T (2) 
isa T (3) 
```

---

# De-_experiment_-ed features:

* signatures
  * experimental since Perl 5.20 (2014)
  * Some changes in the meantime
  * "stable" now
  * does not include named variables
  * using `@_` in signatured subroutines triggers a warning

---

# De-_experiment_-ed features:

* mandatory positional parameters

```perl [1,5-7|3]
use feature 'signatures';

event( 'gpw', 2022 );

sub event ( $name, $year ) {
    say sprintf "Welcome to %s %s", $name, $year;
}
```

```
$ perl mandatory_positional.pl 
Welcome to gpw 2022
```

---

# De-_experiment_-ed features:

* mandatory positional parameters

```perl [1,5-7|3]
use feature 'signatures';

event( 'gpw' );

sub event ( $name, $year ) {
    say sprintf "Welcome to %s %s", $name, $year;
}
```

```
$ perl mandatory_positional.pl 
Too few arguments for subroutine 'main::event' (got 1; expected 2) at mandatory_positional.pl line 3.
```

---

# De-_experiment_-ed features:

* optional positional parameters
    * default values

```perl [5|]
use feature 'signatures';

event( 'gpw' );

sub event ( $name, $year = 2022 ) {
    say sprintf "Welcome to %s %s", $name, $year;
}
```

```
$ perl optional_default1.pl 
Welcome to gpw 2022
```

---

# De-_experiment_-ed features:

* optional positional parameters
    * default value

```perl [5,7,9-11|]
use feature 'signatures';

event( 'gpw' );

sub event ( $name, $year = do { current_year() } ) {
    say sprintf "Welcome to %s %s", $name, $year;
}

sub current_year () {
    (localtime)[5] + 1900;
}
```

```
$ perl optional_default2.pl 
Welcome to gpw 2022
```

---

# De-_experiment_-ed features:

* mandatory positional parameters

```perl [1,5-7|3]
use feature 'signatures';

event( 'gpw', 2022, 2023 );

sub event ( $name, $year ) {
    say sprintf "Welcome to %s %s", $name, $year;
}
```

```
$ perl mandatory_positional.pl 
Too many arguments for subroutine 'main::event' (got 3; expected 2) at mandatory_positional.pl line 3.
```

---

# De-_experiment_-ed features:

* Slurpy parameters
    * default values

```perl [6,10|3,4|]
use feature 'signatures';

event( 'gpw', 2022 );
event( 'gpw', 2022, 1999 .. 2021 );

sub event ( $name, $year, @more_years ) {
    say sprintf "Welcome to %s %s", $name, $year;

    say "Past years: @more_years" if @more_years
}
```

```
$ perl slurpy.pl 
Welcome to gpw 2022
Welcome to gpw 2022
Past years: 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009 2010 2011 2012 2013 2014 2015 2016 2017 2018 2019 2020 2021
```

---

# De-_experiment_-ed features:

| Parameters | Signature |
|----------------------------------------------------|---------------------------------------|
| Exactly zero arguments - empty signature | () |
| Exactly one named argument | ($cat) |
| Zero or one arguments, with no default, unnamed | ($=) |
| Zero or one arguments, with no default, named | ($cat=) |
| Zero or one arguments, with a default | ($cat='Buster') |
| Exactly two arguments |  ($cat, $dog) |
| Two or more arguments, any number | ($cat, $dog, @) |

---

# De-_experiment_-ed features:

| Parameters | Signature |
|----------------------------------------------------|---------------------------------------|
| Two or more arguments, but an even number | ($cat, $dog, %) |
| Two or more arguments, slurping rest (array) | ($cat, $dog, @animals) |
| Two or more arguments, slurping rest (hash) | ($cat, $dog, %animals) |
| Two, three, or four arguments, no defaults | ($cat, $dog, $=, $=) |
| Two, three, or four arguments, one default | ($cat, $dog, $lizard='Godzilla', $=) |
| Class method | ( $class, ... ) |
| Object method | ( $self, ... ) |

Source: https://www.effectiveperlprogramming.com/2015/04/use-v5-20-subroutine-signatures/

------

# various other stuff

* Unicode 14.0 is supported
* _SIGFPE_ is no longer defered

------

# `defer` blocks

* define a block anywhere, run it when leaving the scope
* esp. nice to define teardown activities

---

# `defer` blocks

```perl [1|3,8|4|5|7]
use feature 'defer';
 
{
    say "This happens first";
    defer { say "This happens last"; }
 
    say "And this happens inbetween";
}
```

```
$ perl 01.pl 
defer is experimental at 01.pl line 5.
This happens first
And this happens inbetween
This happens last
```

---

# `defer` blocks

```perl [1|3,11|4,5|7,8]
use feature 'defer';

{
   my $dbh = DBI->connect( ... ) or die "Cannot connect";
   defer { $dbh->disconnect; }
 
   my $sth = $dbh->prepare( ... ) or die "Cannot prepare";
   defer { $sth->finish; }
 
   #...
}
```

------

# Iteration over multiple values at a time

* get n elements from a list in one run

```perl
for my ($key, $value) (%hash) { ... }
for my ($left, $right, $gripping) (@moties) { ... }
```

------

# Stable boolean tracking

* Allows `isbool` to check if a value is of boolean nature
* useful when interoperating with other languages

---

# Stable boolean tracking

```perl [3,4|6,7]
use Devel::Peek;

my $var = 1;
Dump( $var );

my $is_empty = !-s __FILE__;
Dump( $is_empty );
```

---

# Stable boolean tracking

## Perl 5.32

``` [1-4|5-12]
SV = IV(0x556ae8182630) at 0x556ae8182640
  REFCNT = 1
  FLAGS = (IOK,pIOK)
  IV = 1
SV = PVNV(0x556ae815d380) at 0x556ae81825f8
  REFCNT = 1
  FLAGS = (IOK,NOK,POK,pIOK,pNOK,pPOK)
  IV = 0
  NV = 0
  PV = 0x556ae8202560 ""\0
  CUR = 0
  LEN = 10
```

---

# Stable boolean tracking

## Perl 5.36

``` [1-4|5-12]
SV = IV(0x56048e6a1ab0) at 0x56048e6a1ac0
  REFCNT = 1
  FLAGS = (IOK,pIOK)
  IV = 1
SV = PVNV(0x56048e67c380) at 0x56048e6a1a78
  REFCNT = 1
  FLAGS = (IOK,NOK,POK,IsCOW,pIOK,pNOK,pPOK)
  IV = 0
  NV = 0
  PV = 0x56048cfc87e3 "" [BOOL PL_No]
  CUR = 0
  LEN = 0
```

---

# Stable boolean tracking

```perl
use v5.35;

my $var = 1;
my $is_empty = !-s __FILE__;

use builtin 'isbool';
say isbool( $var );
say isbool( $is_empty );
```

```
$ perl boolean/tracking.pl 

1
```

------

# `builtin::` Functions

* A new pragma to import built-in utility functions
  * true, false, isbool
  * weaken, unweaken, isweak
  * blessed
  * refaddr, reftype
  * ceil, floor
  * trim

---

# `builtin::` Functions

```perl [3,4|8,9|11,12,14]
use v5.35;

use builtin qw(true false isbool);
no warnings 'experimental::builtin';

use Data::Printer;

my $true = true;
my $false = false;

p $true;
p $false;

say isbool( $true );
```

```
$ perl bool.pl 
1
""
1
```

---

# `builtin::` Functions

```perl
use v5.35;

use builtin qw(true false);
no warnings 'experimental::builtin';

test("good morning");

sub test ( $greeting, $uppercase = true ) {
    say sprintf "%s Leipzig", ( $uppercase ? uc $greeting : $greeting );
}
```

```
GOOD MORNING Leipzig
```

---

# `builtin::` Functions

```perl
use v5.35;

use builtin qw(true false);
no warnings 'experimental::builtin';

test("good morning", false);

sub test ( $greeting, $uppercase = true ) {
    say sprintf "%s Leipzig", ( $uppercase ? uc $greeting : $greeting );
}
```

```
good morning Leipzig
```

---

# `builtin::` Functions

```perl [3,8|5,6|10-12]
use v5.35;

use Math::BigInt;

use builtin qw(blessed);
no warnings 'experimental::builtin';

my $int = Math::BigInt->new(2022);

if ( blessed $int ) {
    say "\$int is blessed";
}       
```

```
$ perl blessed.pl 
$int is blessed
```

---


# `builtin::` Functions

```
$ perl -E 'say ">>", builtin::trim(" das ist ein Test!     "), "<<";'
Built-in function 'builtin::trim' is experimental at -e line 1.
>>das ist ein Test!<<
```

------

# Questions?

Slides (as markdown source) and demo code:

- <https://os.perl-services.de/perl-academy/talks/-/tree/main/2022/gpw>

Slideshow:

- <https://talks.perl-academy.de/2022/gpw/>

------

# Thank you!
