# About me
  
* Renée Bäcker
  * Owner of <a href="https://perl-services.de">Perl-Services.de</a> (<a href="https://metacpan.org/author/PERLSRVDE">PERLSRVDE @ cpan</a>)
  * <a href="https://perl-academy.de">Perl-Academy.de</a>
  * <a href="https://metacpan.org/author/RENEEB">RENEEB @ cpan</a>
  * Interests: Perl, IT-Security

------

# Links

  * This talk is already available at [https://talks.perl-academy.de](https://talks.perl-academy.de)
  * Code is available at [https://os.perl-services.de/perl-academy](https://os.perl-services.de/perl-academy)

------

# Perl 5.38

  * very small changes (mostly bugfixes, doc changes, updated modules)
  * some nice (but small additions)
  * but ... (we'll come to that later)

---

# Perl 5.38

  * likely to be released in May/June 2023
  * features may change

------

# no _true_ value needed for modules

```perl
# < 5.38
package Perl536;

# ... code ...

1;
```
---

# no _true_ value needed for modules

```perl
# < 5.38
package Perl536;

# ... code ...
# no true value
```

---

# no _true_ value needed for modules

```
$ perl -I. -MPerl536 -E 1
Perl536.pm did not return a true value.
BEGIN failed--compilation aborted.
```

---

# no _true_ value needed for modules

```perl
package Perl538;

use v5.37.8;
# ... code ...
```

---

# no _true_ value needed for modules

```perl
package Perl538;

use feature "module_true";
# ... code ...
```

---

# no _true_ value needed for modules

```
$ perl -I. -MPerl538 -E 1
$
```

------

# Changes to subroutine signatures

```perl
#!/usr/bin/perl
  
use v5.37.8;
use experimental 'signatures';

test();
test( 'GPW' );
test( undef );

sub test ( $name = 'World' ) {
    say "Hello $name!";
}
```

---

# Changes to subroutine signatures

```text
$ perl signatures.pl 
Hello World!
Hello GPW!
Use of uninitialized value $name in concatenation (.) or string at signatures.pl line 11.
Hello !
```

---

# Changes to subroutine signatures

Now

  * `//=`
  * `||=`

are allowed in subroutine signatures

---

# Changes to subroutine signatures

```perl
#!/usr/bin/perl
  
use v5.37.8;
use experimental 'signatures';

test();
test( 'GPW' );
test( undef );

sub test ( $name //= 'World' ) {
    say "Hello $name!";
}
```

---

# Changes to subroutine signatures

```text
$ perl signatures2.pl 
Hello World!
Hello GPW!
Hello World!
```

------

# Use of `'` as package separator is deprecated

  * Who has used `'` as a package separator? As in

*Don/t.pm*

```perl
package Don't;

sub do_it { print "Don't do it\n" };

1;
```

---

# Use of `'` as package separator is deprecated

  * Who has used `'` as a package separator? As in

```perl
use lib '.';
use Don't;

Don't->do_it;
```

---

# Use of `'` as package separator is deprecated

With _Perl <= 5.36_

```perl
$ perl dont.pl 
Don't do it
```

---

# Use of `'` as package separator is deprecated

With _Perl >= 5.38_

```perl
$ perl dont.pl 
Old package separator "'" deprecated at dont.pl line 7.
Old package separator "'" deprecated at Don/t.pm line 1.
Old package separator "'" deprecated at dont.pl line 9.
Don't do it
```

------

# New functions in `builtin`

There are two more functions in the `builtin` package:

  * `is_tainted`
  * `export_lexically`

---

# New functions in `builtin`

  * `true`, `false`, `is_bool`
  * `weaken`, `unweaken`, `is_weak`
  * `blessed`, `refaddr`, `reftype`
  * `created_as_string`, `created_as_number`
  * `ceil`, `floor`
  * `indexed`
  * `trim`

---

# `is_tainted`

```perl[5-8|]
#!/usr/bin/perl
  
use v5.37.8;

use experimental 'builtin';

my $var = <STDIN>;
say builtin::is_tainted($var);
```

---

# `is_tainted`

```text
$ perl -T tainted.pl 
Test
1
```

---

# `export_lexically`

_Traditional_ exports:

```perl[5-8|]
package ExportOld;
  
use v5.37.8;

use base 'Exporter';

our @EXPORT = qw(hello);
sub hello { say 'Hello World!' };
```

---

# `export_lexically`

```perl[3|5,6|]
#!/usr/bin/perl
  
use ExportOld;

hello();
main->hello();
```

```text
$ perl export_old.pl 
Hello World!
Hello World!
```

---

# `export_lexically`

_Lexical_ exports

```perl[6-8|]
package ExportLex;
  
use v5.37.8;

use experimental 'builtin';
use builtin 'export_lexically';

export_lexically hello => sub { say 'Hello World!' };
```

---

# `export_lexically`

```perl[3|5,6|]
#!/usr/bin/perl
  
use ExportLex;

hello();
main->hello();
```

```text
$ perl export.pl 
Hello World!
Can't locate object method "hello" via package "main" at export.pl line x.
```

---

# `export_lexically`

```perl
package ExportLexVar;
  
use v5.37.8;

use experimental 'builtin';
use builtin 'export_lexically';

my ($scalar, @array, %hash);
export_lexically 
  '$year'  => \$scalar,
  '@talks' => \@array,
  '%info'  => \%hash,
  check    => sub {
      say "Year: $scalar, Talks: @array"
  }
;
```

---

# `export_lexically`

```perl
#!/usr/bin/perl
  
use v5.37.8;

use ExportLexVar;

$year = 2023;
@talks = qw(Perl_5.38 Playwright);
check();
```

```text
$ perl export_var.pl 
Year: 2023, Talks: Perl_5.38 Playwright
```

---

# `export_lexically`

```perl
package ExportLexAlias;
  
use v5.37.8;

use experimental 'builtin';
use builtin 'export_lexically';

my ($scalar, @array, %hash);

sub import {
    shift @_;
    my %alias = @_;

    export_lexically
        '$' . ( $alias{year} || 'year' )  => \$scalar,
        check => sub { say "Year: $scalar" }
    ;
}
```

---

# `export_lexically`

```perl
#!/usr/bin/perl
  
use v5.37.8;

use lib '.';
use ExportLexAlias ( year => 'jahr' );

$jahr = 2023;
check();
```

```text
$ perl export_alias.pl 
Year: 2023
```

------

# feature 'class'

A new **experimental** feature: *class*

---

# experimental class

The new native class mechanism is experimental and some features
are missing. But it works very well...

---

# A very simple class

```perl
use v5.37.8;
  
use experimental 'class';

class GPW;
```

  * it already has a constructor

---

# Using a new class

```perl
use lib '.';
use experimental 'builtin';

use GPW;
my $gpw = GPW->new;
say builtin::blessed( $gpw );
say builtin::reftype( $gpw );
```

```text
$ perl gpw.pl 
GPW
OBJECT
```

---

# A simple class

  * _`class`_ [VERSION];
  * _`class`_ [VERSION] `{}`

---

# experimental class

```perl[1-3|7-9|]
use v5.37.8;
  
use experimental 'class';

class GPW 0.01;
```

---

# `class` with block

```perl[1-3|7-9|]
use v5.37.8;
  
use experimental 'class';

class GPW 0.01 {
}
```

---

# `field`s

  * Attributes are `field`s
  * Only accessible within `method`s and `ADJUST` blocks

```perl
class GPW 0.01 {
    field $year;
}
```

---

# `field`s

They can have defaults...

```perl
class GPW 0.01 {
    use Time::Piece;

    field $year = localtime->year;
}
```

---

# `field`s

They can be defined as parameters for `new`...

```perl
class GPW 0.01 {
    field $year :param;
}
```

---

# `field`s

They can be defined as parameters for `new` that can also have defaults...

```perl
class GPW 0.01 {
    use Time::Piece;

    field $year :param = localtime->year;
}
```

---

# `field`s

They can be defined as parameters for `new` that can also have defaults...

```perl
class GPW 0.01 {
    use Time::Piece;

    field $year :param = localtime->year;

    method print_year {
        say "Year: $year";
    }
}
```

---

# `field`s

```perl[3|4|]
use v5.37.8;
use GPW;

GPW->new->print_year;
GPW->new( year => 2019 )->print_year;
```

```text
$ perl gpw.pl 
Year: 2023
Year: 2019
```

---

# `field`s

  * you can define aliases for the constructor parameters

```perl
    field $y :param(year);
```

---

# `field`s

  * parameters without default value are mandatory

```
$ perl gpw.pl 
Required parameter 'year' is missing for "GPW" constructor at gpw.pl line 8.
```

---

# `field`s

  * unknown parameters warn

```perl
GPW->new(year => 2019, city => 'FFM')->print_year;
```

```text
$ perl gpw.pl 
Unrecognised parameters for "GPW" constructor: city at gpw.pl line 9.
```

---

# `method`s

  * similar to `sub`s
  * keyword `method`
  * `$self` is auto-populated (in lexical scope)
  * signatures are there (no need to enable that feature)

---

# `method`s

```perl
class GPW 0.01 {
    field $y :param(year);
    
    method print_year ( $year = $y ) {
        say "Year: $year";
    }
    
    method test1 ($y) {
        $self->print_year( $y );
    }
}

```

---

# `ADJUST` blocks

  * `ADJUST` blocks are called
    * during object construction
    * in order of declaration
  * Have access to the fields
  * ... and `$self`

---

# `ADJUST` blocks

```perl
class GPW 0.01 {
    use Time::Piece;
    use Data::Printer;

    field $year :param = localtime->year;
    field $ua = Mojo::UserAgent->new;
    field @talks;

    ADJUST {
        my $base_url = 'https://act.yapc.eu/gpw' . $year;
        my $tx       = $ua->get( $base_url . '/talks' );
        @talks       = ...; # get talks from HTML
    }

    method print_infos {
        p $year;
        p @talks;
    }
}
```

---

# inheritance

  * only single-inheritance
  * `:isa` class attribute

---

# inheritance

```perl
class GPW 0.01 :isa( Base::Class 3.01 ) { ... }
```

---

# What's missing

  * syntax to declare/consume a role
  * `ADJUST` blocks
    * Parameters
    * real blocks
  * Accessor generator for attributes
  * Metaprogramming

------

# Have fun

... and enjoy Perl 5.38!

